﻿using combit.Reporting;
using Microsoft.Win32;
using s2industries.ZUGFeRD;
using System.IO;
using System.Windows;


namespace LlZUGFeRDExporter
{
    public partial class MainWindow : Window
    {
        string _projectPath;

        public MainWindow()
        {
            InitializeComponent();
            _projectPath = System.AppDomain.CurrentDomain.BaseDirectory + @"\..\..\..\Invoice.lst";

            // Check if the List & Label file exists
            if (!File.Exists(_projectPath))
            {
                MessageBox.Show($"Report file '{_projectPath}' does not exist!");
                return;
            }
        }

        private void Button_Export(object sender, RoutedEventArgs e)
        {
            string xmlFileName = "ZUGFeRD-invoice.xml";
            string xmlFilePath = Path.Combine(Path.GetDirectoryName(_projectPath),xmlFileName);

            // Export the invoice to XML
            ExportInvoiceToXml(GetInvoiceDescriptor(), xmlFilePath);

            // Export PDF with ZUGFeRD
            ExportPdfWithZUGFeRD(xmlFilePath);
        }

        // Method to get Invoice descriptor
        private InvoiceDescriptor GetInvoiceDescriptor()
        {
            InvoiceDescriptor desc = InvoiceDescriptor.CreateInvoice("12345", new DateTime(2024, 01, 03), CurrencyCodes.EUR);
            desc.AddNote("Rechnung gemäß Bestellung vom 03.01.2024.");
            desc.AddNote(note: "Muster GmbH\r\nLieferantenstraße 20\r\n78315 Radolfzell\r\nDeutschland\r\nGeschäftsführer: Max Mustermann\r\nHandelsregisternummer: H A 123\r\n",
                         subjectCode: SubjectCodes.REG
                        );

            desc.AddTradeLineItem(name: "Schuhe",
                                  unitCode: QuantityCodes.H87,
                                  sellerAssignedID: "TB100A4",
                                  id: new GlobalID(GlobalIDSchemeIdentifiers.EAN, "4012345001235"),
                                  grossUnitPrice: 90.99m,
                                  netUnitPrice: 90.99m,
                                  billedQuantity: 20m,
                                  taxType: TaxTypes.VAT,
                                  categoryCode: TaxCategoryCodes.S,
                                  taxPercent: 19m
                                 );

            desc.AddTradeLineItem(name: "T-Shirt",
                                  unitCode: QuantityCodes.H87,
                                  sellerAssignedID: "ARNR2",
                                  id: new GlobalID(GlobalIDSchemeIdentifiers.EAN, "4000050986428"),
                                  grossUnitPrice: 50m,
                                  netUnitPrice: 50m,
                                  billedQuantity: 50,
                                  taxType: TaxTypes.VAT,
                                  categoryCode: TaxCategoryCodes.S,
                                  taxPercent: 19m
                                 );

            desc.ReferenceOrderNo = "04011000-12345-34";
            desc.SetSeller(name: "Muster GmbH",
                           postcode: "78315",
                           city: "Radolfzell",
                           street: "Lieferantenstraße 20",
                           country: CountryCodes.DE,
                           id: "",
                           globalID: new GlobalID(GlobalIDSchemeIdentifiers.GLN, "4000001123452"),
                           legalOrganization: new LegalOrganization(GlobalIDSchemeIdentifiers.GLN, "4000001123452", "Lieferant GmbH")
                          );

            desc.SetSellerContact(name: "Max Mustermann",
                                  orgunit: "Muster-Einkauf",
                                  emailAddress: "Max@Mustermann.de",
                                  phoneno: "+49891234567"
                                 );

            desc.AddSellerTaxRegistration("201/113/40209", TaxRegistrationSchemeID.FC);
            desc.AddSellerTaxRegistration("DE123456789", TaxRegistrationSchemeID.VA);

            desc.SetBuyer(name: "Muster Einkauf GmbH",
                          postcode: "78224",
                          city: "Singen",
                          street: "Musterstraße 15",
                          country: CountryCodes.DE,
                          id: "GE2020211"
                         );

            desc.ActualDeliveryDate = new DateTime(2024, 01, 03);
            desc.SetPaymentMeans(PaymentMeansTypeCodes.SEPACreditTransfer, "Zahlung per SEPA Überweisung.");
            desc.AddCreditorFinancialAccount(iban: "DE02120300000000202051", bic: "BYLADEM1001", name: "Kunden");

            desc.AddApplicableTradeTax(basisAmount: 275.0m,
                                       percent: 7m,
                                       typeCode: TaxTypes.VAT,
                                       categoryCode: TaxCategoryCodes.S
                                      );

            desc.AddApplicableTradeTax(basisAmount: 198.0m,
                                       percent: 19m,
                                       typeCode: TaxTypes.VAT,
                                       categoryCode: TaxCategoryCodes.S
                                      );

            desc.SetTradePaymentTerms("Zahlbar innerhalb 30 Tagen netto bis 04.02.2024, 3% Skonto innerhalb 10 Tagen bis 13.01.2024");
            desc.SetTotals(lineTotalAmount: 473.0m,
                           taxBasisAmount: 473.0m,
                           taxTotalAmount: 56.87m,
                           grandTotalAmount: 529.87m,
                           duePayableAmount: 529.87m
                          );

            return desc;
        }

        // Method to export invoice to XML 
        internal void ExportInvoiceToXml(InvoiceDescriptor invoice, string filePath)
        {
            //Create a file stream
            FileStream stream = new FileStream(filePath, FileMode.Create, FileAccess.Write);

            // Save the invoice to XML
            invoice.Save(stream, ZUGFeRDVersion.Version21, Profile.Basic);
            stream.Flush();
            stream.Close();
        }

        // Method to export to PDF with ZUGFeRD options
        private void ExportPdfWithZUGFeRD(string xmlFilePath)
        {
            string pdfFileName = "Export.pdf";
            string pdfFilePath = Path.Combine(Path.GetDirectoryName(_projectPath), pdfFileName);

            ExportConfiguration exportConfiguration = new ExportConfiguration(LlExportTarget.Pdf, pdfFilePath, _projectPath);
            exportConfiguration.ShowResult = true;

            using (ListLabel LL = new ListLabel())
            {
                LL.ExportOptions.Add(LlExportOption.PdfZUGFeRDVersion, "2.1");
                LL.ExportOptions.Add(LlExportOption.PdfZUGFeRDConformanceLevel, "BASIC");
                LL.ExportOptions.Add(LlExportOption.PdfZUGFeRDXmlPath, xmlFilePath);
                var desc = GetInvoiceDescriptor();
                var dataSource = new List<InvoiceDescriptor?>();
                dataSource.Add(desc);

                LL.DataSource = dataSource;
                LL.DataMember = "InvoiceDescriptor";
                LL.AutoMasterMode = LlAutoMasterMode.AsVariables;

                // Start export
                LL.Export(exportConfiguration);
            }
        }

        private void Button_Design(object sender, RoutedEventArgs e)
        {
            using (ListLabel LL = new ListLabel())
            {
                // Set up invoice descriptor and data source
                var desc = GetInvoiceDescriptor();
                var dataSource = new List<InvoiceDescriptor?>();
                dataSource.Add(desc);

                // Configure List & Label
                LL.DataSource = dataSource;
                LL.DataMember = "InvoiceDescriptor";
                LL.AutoMasterMode = LlAutoMasterMode.AsVariables;
                LL.AutoProjectFile = _projectPath;
                LL.AutoShowSelectFile = false;

                // Open the List & Label designer
                LL.Design();
            }
        }
    }
}